package massim.agent.framework;

import cz.agents.alite.communication.DefaultCommunicator;
import cz.agents.alite.communication.channel.CommunicationChannelException;
import cz.agents.alite.communication.channel.DefaultCommunicationChannelBroadcast;
import cz.agents.alite.communication.channel.DirectCommunicationChannelAsync;
import cz.agents.alite.communication.protocol.simple.SimpleProtocol;

public abstract class AbstractALiteAgent implements Runnable {

	private static String MASSIM_PROTOCOL_NAME = "MASSimProtocol";
	protected static String BROADCAST_ADDRESS = DefaultCommunicationChannelBroadcast.BROADCAST_ADDRESS;
	
	private SimpleProtocol comm;
	private DefaultCommunicator communicator;

	public AbstractALiteAgent(String id) {

		communicator = new DefaultCommunicator(id);

		try {
			communicator.addChannel(new DirectCommunicationChannelAsync(communicator));
		} catch (CommunicationChannelException e) {
			e.printStackTrace();
		}
		
		comm = new SimpleProtocol(communicator, MASSIM_PROTOCOL_NAME) {
			
			@Override
			protected void processMessage(Object content) {
				onMessageReceived((MessageContent) content);
			}
		};
	}

	protected abstract void onMessageReceived(MessageContent content);
	
	private SimpleProtocol getComm() {
		return comm;
	}

	// public send message facility
	public void sendMessage(MessageContent content, String receiver) {
		getComm().sendMessage(content.clone(), receiver);
	}

	// public broadcast facility
	public void broadcastMessage(MessageContent content ) {
		getComm().sendMessage(content, BROADCAST_ADDRESS);
	}
}
